@@ -1,14 +1,13 @@
 /*!
-Compact Position Reporting
-
-This module turns an ADS-B CPR format into Latitude/Longitude: `Position`.
+Compact Position Reporting for [`Position`] Reporting
 
 reference: ICAO 9871 (D.2.4.7)
 !*/
 
-use crate::{Altitude, CPRFormat};
 use std::cmp;
 
+use crate::{Altitude, CPRFormat};
+
 const NZ: f64 = 15.0;
 const D_LAT_EVEN: f64 = 360.0 / (4.0 * NZ);
 const D_LAT_ODD: f64 = 360.0 / (4.0 * NZ - 1.0);
@@ -218,27 +217,21 @@ pub fn get_position(cpr_frames: (&Altitude, &Altitude)) -> Option<Position> {
     let latest_frame = cpr_frames.1;
     let (even_frame, odd_frame) = match cpr_frames {
         (
-            even
-            @
-            Altitude {
+            even @ Altitude {
                 odd_flag: CPRFormat::Even,
                 ..
             },
-            odd
-            @ Altitude {
+            odd @ Altitude {
                 odd_flag: CPRFormat::Odd,
                 ..
             },
         )
         | (
-            odd
-            @ Altitude {
+            odd @ Altitude {
                 odd_flag: CPRFormat::Odd,
                 ..
             },
-            even
-            @
-            Altitude {
+            even @ Altitude {
                 odd_flag: CPRFormat::Even,
                 ..
             },
