@@ -1,7 +1,10 @@
+#![doc(
+    html_logo_url = "https://raw.githubusercontent.com/wcampbell0x2a/adsb_deku/master/media/logo.png"
+)]
 /*!
 `adsb_deku` provides decoding for the [`ADS-B`] Downlink protocol by using the [`deku`] crate.
 
-# Support
+## Downlink Format support
 |  [`DF`]  |  Name                               |  Section    |
 | -------- | ----------------------------------- | ----------- |
 | 0        | [`Short Air-Air Surveillance`]      | 3.1.2.8.2   |
@@ -10,12 +13,38 @@
 | 11       | [`All Call Reply`]                  | 2.1.2.5.2.2 |
 | 16       | [`Long Air-Air Surveillance`]       | 3.1.2.8.3   |
 | 17       | [`Extended Squitter(ADS-B)`]        | 3.1.2.8.6   |
-| 18       | [`Extended Squitter(Supplementary)`]| 3.1.2.8.7   |
+| 18       | [`Extended Squitter(TIS-B)`]        | 3.1.2.8.7   |
 | 19       | [`Extended Squitter(Military)`]     | 3.1.2.8.8   |
 | 20       | [`Comm-B Altitude Reply`]           | 3.1.2.6.6   |
 | 21       | [`Comm-B Identity Reply`]           | 3.1.2.6.8   |
 | 24       | [`Comm-D`]                          | 3.1.2.7.3   |
 
+# [`Comm-B Altitude Reply`] and [`Comm-B Identity Reply`] Comm-B Support
+
+|  [`BDS`]  |  Name                                   |  Table      |
+| --------- | --------------------------------------- | ----------- |
+| (0,0)     | [`Empty`]                               |             |
+| (1,0)     | [`Data Link Capability`]                | A-2-16      |
+| (2,0)     | [`Aircraft Identification`]             | A-2-32      |
+
+# [`Extended Squitter(ADS-B)`] and [`Extended Squitter(TIS-B)`] Type Code support
+
+|  [`ME`](Type Code)  |  Name                                  |
+| ------------------- | -----------------------------------    |
+| 0                   | [`ME::NoPosition`]                     |
+| 1..=4               | [`ME::AircraftIdentification`]         |
+| 5..=8               | [`ME::SurfacePosition`]                |
+| 9..=18              | [`ME::AirbornePositionBaroAltitude`]   |
+| 19                  | [`ME::AirborneVelocity`]               |
+| 20..=22             | [`ME::AirbornePositionGNSSAltitude`]   |
+| 23                  | [`ME::Reserved0`]                      |
+| 24                  | [`ME::SurfaceSystemStatus`]            |
+| 25..=27             | [`ME::Reserved1`]                      |
+| 28                  | [`ME::AircraftStatus`]                 |
+| 29                  | [`ME::TargetStateAndStatusInformation`]|
+| 30                  | [`ME::AircraftOperationalCoordination`]|
+| 31                  | [`ME::AircraftOperationStatus`]        |
+
 # Example
 To begin using `adsb_deku`, import the [`Frame`] struct as well as the trait [`deku::DekuContainerRead`].
 This trait is re-exported for your convenience. [`Frame::from_bytes()`] provides the interface for decoding bytes
@@ -35,7 +64,6 @@ assert_eq!(
   Altitude:      23650 ft barometric
   CPR type:      Airborne
   CPR odd flag:  even
-  CPR NUCp/NIC:  ?
   CPR latitude:  (87769)
   CPR longitude: (71577)
 "#,
@@ -47,19 +75,38 @@ assert_eq!(
 The [`apps/`] directory of the project repository contains programs `radar` and `1090` for showcasing
 different `adsb_deku` uses. See the [`README.md`] for examples of use.
 
-
 [`DF`]: crate::DF
+[`ME`]: crate::adsb::ME
+[`BDS`]: crate::bds::BDS
 [`Short Air-Air Surveillance`]: crate::DF::ShortAirAirSurveillance
 [`Surveillance Altitude Reply`]: crate::DF::SurveillanceAltitudeReply
 [`Surveillance Identity Reply`]: crate::DF::SurveillanceIdentityReply
 [`All Call Reply`]: crate::DF::AllCallReply
 [`Long Air-Air Surveillance`]: crate::DF::LongAirAir
 [`Extended Squitter(ADS-B)`]: crate::DF::ADSB
-[`Extended Squitter(Supplementary)`]: crate::DF::TisB
+[`Extended Squitter(TIS-B)`]: crate::DF::TisB
 [`Extended Squitter(Military)`]: crate::DF::ExtendedQuitterMilitaryApplication
 [`Comm-B Altitude Reply`]: crate::DF::CommBAltitudeReply
 [`Comm-B Identity Reply`]: crate::DF::CommBIdentityReply
 [`Comm-D`]: crate::DF::CommDExtendedLengthMessage
+
+[`Empty`]: crate::bds::BDS::Empty
+[`Data Link Capability`]: crate::bds::BDS::DataLinkCapability
+[`Aircraft Identification`]: crate::bds::BDS::AircraftIdentification
+[`ME::NoPosition`]: crate::adsb::ME::NoPosition
+[`ME::AircraftIdentification`]: crate::adsb::ME::AircraftIdentification
+[`ME::SurfacePosition`]: crate::adsb::ME::SurfacePosition
+[`ME::AirbornePositionBaroAltitude`]: crate::adsb::ME::AirbornePositionBaroAltitude
+[`ME::AirborneVelocity`]: crate::adsb::ME::AirborneVelocity
+[`ME::AirbornePositionGNSSAltitude`]: crate::adsb::ME::AirbornePositionGNSSAltitude
+[`ME::Reserved0`]: crate::adsb::ME::Reserved0
+[`ME::SurfaceSystemStatus`]: crate::adsb::ME::SurfaceSystemStatus
+[`ME::Reserved1`]: crate::adsb::ME::Reserved1
+[`ME::AircraftStatus`]: crate::adsb::ME::AircraftStatus
+[`ME::TargetStateAndStatusInformation`]: crate::adsb::ME::TargetStateAndStatusInformation
+[`ME::AircraftOperationalCoordination`]: crate::adsb::ME::AircraftOperationalCoordination
+[`ME::AircraftOperationStatus`]: crate::adsb::ME::AircraftOperationStatus
+
 [`apps`]: crate::Frame
 [`Frame`]: crate::Frame
 [`deku::DekuContainerRead`]: crate::deku::DekuContainerRead
@@ -69,22 +116,28 @@ different `adsb_deku` uses. See the [`README.md`] for examples of use.
 [`deku`]: https://github.com/sharksforarms/deku
 !*/
 
+// good reference: http://www.anteni.net/adsb/Doc/1090-WP30-18-DRAFT_DO-260B-V42.pdf
+//
+// Maybe always reference this in the future?
+
 /// re-export deku
 pub use deku;
 
 pub mod adsb;
+pub mod bds;
 pub mod cpr;
 mod crc;
+mod mode_ac;
 
 use adsb::{ControlField, ADSB};
-
+use bds::BDS;
 use deku::bitvec::{BitSlice, Msb0};
 use deku::prelude::*;
 
 /// Downlink ADS-B Packet
 #[derive(Debug, PartialEq, DekuRead, Clone)]
 pub struct Frame {
-    /// 5 bit identifier
+    /// Starting with 5 bit identifier, decode packet
     pub df: DF,
     /// Calculated from all bits, used as ICAO for Response packets
     #[deku(reader = "Self::read_crc(df, deku::input_bits)")]
@@ -128,7 +181,7 @@ impl std::fmt::Display for Frame {
                 } else {
                     writeln!(f, "  Air/Ground:    ground")?;
                 }
-            }
+            },
             DF::SurveillanceAltitudeReply { fs, ac, .. } => {
                 writeln!(f, " Surveillance, Altitude Reply")?;
                 writeln!(f, "  ICAO Address:  {:06x} (Mode S / ADS-B)", self.crc)?;
@@ -136,20 +189,20 @@ impl std::fmt::Display for Frame {
                 if ac.0 > 0 {
                     writeln!(f, "  Altitude:      {} ft barometric", ac.0)?;
                 }
-            }
+            },
             DF::SurveillanceIdentityReply { fs, id, .. } => {
                 writeln!(f, " Surveillance, Identity Reply")?;
                 writeln!(f, "  ICAO Address:  {:06x} (Mode S / ADS-B)", self.crc)?;
                 writeln!(f, "  Air/Ground:    {}", fs)?;
                 writeln!(f, "  Identity:      {:04x}", id.0)?;
-            }
+            },
             DF::AllCallReply {
                 capability, icao, ..
             } => {
                 writeln!(f, " All Call Reply")?;
                 writeln!(f, "  ICAO Address:  {} (Mode S / ADS-B)", icao)?;
                 writeln!(f, "  Air/Ground:    {}", capability)?;
-            }
+            },
             DF::LongAirAir { altitude, .. } => {
                 writeln!(f, " Long Air-Air ACAS")?;
                 writeln!(f, "  ICAO Address:  {:06x} (Mode S / ADS-B)", self.crc)?;
@@ -160,42 +213,39 @@ impl std::fmt::Display for Frame {
                 } else {
                     writeln!(f, "  Air/Ground:    ground")?;
                 }
-            }
+            },
             DF::ADSB(adsb) => {
                 write!(f, "{}", adsb.to_string("(Mode S / ADS-B)").unwrap())?;
-            }
+            },
             DF::TisB { cf, .. } => {
                 write!(f, "{}", cf)?;
-            }
+            },
             // TODO
-            DF::ExtendedQuitterMilitaryApplication { .. } => {}
-            DF::CommBAltitudeReply { mb, parity, .. } => {
-                writeln!(f, " Comm-B Altitude Reply")?;
-                writeln!(f, "  data: {:x?}", mb)?;
-                writeln!(f, "  ICAO Address: {} (Mode S / ADS-B)", parity)?;
-            }
-            DF::CommBIdentityReply {
-                id, message_comm_b, ..
-            } => {
+            DF::ExtendedQuitterMilitaryApplication { .. } => {},
+            DF::CommBAltitudeReply { bds, alt, .. } => {
+                writeln!(f, " Comm-B, Altitude Reply")?;
+                writeln!(f, "  ICAO Address:  {:x?} (Mode S / ADS-B)", self.crc)?;
+                writeln!(f, "  Altitude:      {} ft", alt.0)?;
+                write!(f, "  {}", bds)?;
+            },
+            DF::CommBIdentityReply { id, bds, .. } => {
                 writeln!(f, " Comm-B, Identity Reply")?;
-                if message_comm_b.is_empty() {
-                    writeln!(f, "    Comm-B format: unknown format")?;
-                } else {
-                    writeln!(f, "    Comm-B format: {}", message_comm_b)?;
-                }
                 writeln!(f, "    ICAO Address:  {:x?} (Mode S / ADS-B)", self.crc)?;
                 writeln!(f, "    Squawk:        {:x?}", id)?;
-            }
+                write!(f, "    {}", bds)?;
+            },
             DF::CommDExtendedLengthMessage { .. } => {
                 writeln!(f, " Comm-D Extended Length Message")?;
                 writeln!(f, "    ICAO Address:     {:x?} (Mode S / ADS-B)", self.crc)?;
-            }
+            },
         }
         Ok(())
     }
 }
 
 /// Downlink Format (3.1.2.3.2.1.2)
+///
+/// Starting with 5 bits, decode the rest of the message as the correct data packets
 #[derive(Debug, PartialEq, DekuRead, Clone)]
 #[deku(type = "u8", bits = "5")]
 pub enum DF {
@@ -304,9 +354,6 @@ pub enum DF {
     ///
     /// Non-Transponder-based ADS-B Transmitting Subsystems and TIS-B Transmitting equipment.
     /// Equipment that cannot be interrogated.
-    ///
-    /// reference: Aeronautical Telecommunications Volume IV: Surveillance and
-    /// Collision Avoidance Systems, Fifth Edition
     #[deku(id = "18")]
     TisB {
         /// Enum containing message
@@ -324,8 +371,6 @@ pub enum DF {
     },
 
     /// 20: COMM-B Altitude Reply (3.1.2.6.6)
-    ///
-    /// TODO: Test me
     #[deku(id = "20")]
     CommBAltitudeReply {
         /// FS: Flight Status
@@ -335,11 +380,9 @@ pub enum DF {
         /// UM: Utility Message
         um: UtilityMessage,
         /// AC: Altitude Code
-        #[deku(reader = "Altitude::read(deku::rest)")]
-        alt: u32,
+        alt: AC13Field,
         /// MB Message, Comm-B
-        #[deku(count = "7")]
-        mb: Vec<u8>,
+        bds: BDS,
         /// AP: address/parity
         parity: ICAO,
     },
@@ -354,18 +397,14 @@ pub enum DF {
         /// UM: Utility Message
         um: UtilityMessage,
         /// ID: Identity
-        ///
-        /// TODO: does this work?
         #[deku(
             bits = "13",
             endian = "big",
-            map = "|squawk: u32| -> Result<_, DekuError> {Ok(decode_id13_field(squawk))}"
+            map = "|squawk: u32| -> Result<_, DekuError> {Ok(mode_ac::decode_id13_field(squawk))}"
         )]
         id: u32,
-        /// MB Message, COMM-B
-        //#TODO: this works?
-        #[deku(reader = "read_comm_b(deku::rest)")]
-        message_comm_b: String,
+        /// MB Message, Comm-B
+        bds: BDS,
         /// AP address/parity
         parity: ICAO,
     },
@@ -389,23 +428,6 @@ pub enum DF {
     },
 }
 
-fn read_comm_b(rest: &BitSlice<Msb0, u8>) -> Result<(&BitSlice<Msb0, u8>, String), DekuError> {
-    pub const AIS_CHARSET: &str =
-        "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_ !\"#$%&'()*+,-./0123456789:;<=>?";
-    let (rest, ident) = <u8>::read(rest, deku::ctx::Size::Bits(7))?;
-
-    let mut inside_rest = rest;
-    let mut callsign = String::new();
-    if ident == 0x20 {
-        for _ in 0..8 {
-            let (for_rest, c) = <u8>::read(inside_rest, deku::ctx::Size::Bits(5))?;
-            callsign.push(AIS_CHARSET.chars().nth(c as usize).unwrap());
-            inside_rest = for_rest;
-        }
-    }
-    Ok((inside_rest, callsign))
-}
-
 /// [`DF::CommBAltitudeReply`] || ([`DF::ADSB`] && ([`adsb::ME::AirbornePositionBaroAltitude`] || [`adsb::ME::AirbornePositionGNSSAltitude`])
 #[derive(Debug, PartialEq, DekuRead, Default, Copy, Clone)]
 pub struct Altitude {
@@ -430,42 +452,37 @@ pub struct Altitude {
 impl std::fmt::Display for Altitude {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         writeln!(f, "  Altitude:      {} ft barometric", self.alt)?;
-        // TODO: fix me
         writeln!(f, "  CPR type:      Airborne")?;
         writeln!(f, "  CPR odd flag:  {}", self.odd_flag)?;
-        // TODO: fix me
-        writeln!(f, "  CPR NUCp/NIC:  ?")?;
         writeln!(f, "  CPR latitude:  ({})", self.lat_cpr)?;
         writeln!(f, "  CPR longitude: ({})", self.lon_cpr)?;
-        // TODO: fix me
-        //println!("{}", self.t);
         Ok(())
     }
 }
 
 impl Altitude {
+    /// decodeAC12Field
     fn read(rest: &BitSlice<Msb0, u8>) -> Result<(&BitSlice<Msb0, u8>, u32), DekuError> {
         let (rest, num) = u32::read(rest, (deku::ctx::Endian::Big, deku::ctx::Size::Bits(12)))?;
 
         let q = num & 0x10;
 
         if q > 0 {
-            // regular
-            // TODO this is meters?
             let n = ((num & 0x0fe0) >> 1) | (num & 0x000f);
             let n = n * 25;
             if n > 1000 {
                 Ok((rest, (n - 1000) as u32))
             } else {
-                // TODO add error
                 Ok((rest, 0))
             }
         } else {
-            // mode c?
-            // TODO this is feet
-            let n = ((num & 0x0fc0) << 1) | (num & 0x003f);
-            let _altitude = mode_ac::mode_a_to_index(decode_id13_field(n));
-            Ok((rest, ((n as u32) * 100)))
+            let mut n = ((num & 0x0fc0) << 1) | (num & 0x003f);
+            n = mode_ac::decode_id13_field(n);
+            if let Ok(n) = mode_ac::mode_a_to_mode_c(n) {
+                Ok((rest, ((n as u32) * 100)))
+            } else {
+                Ok((rest, (0)))
+            }
         }
     }
 }
@@ -485,51 +502,6 @@ impl Default for SurveillanceStatus {
     }
 }
 
-/// gillham code
-fn decode_id13_field(id13_field: u32) -> u32 {
-    let mut hex_gillham: u32 = 0;
-
-    if id13_field & 0x1000 != 0 {
-        hex_gillham |= 0x0010;
-    } // Bit 12 = C1
-    if id13_field & 0x0800 != 0 {
-        hex_gillham |= 0x1000;
-    } // Bit 11 = A1
-    if id13_field & 0x0400 != 0 {
-        hex_gillham |= 0x0020;
-    } // Bit 10 = C2
-    if id13_field & 0x0200 != 0 {
-        hex_gillham |= 0x2000;
-    } // Bit  9 = A2
-    if id13_field & 0x0100 != 0 {
-        hex_gillham |= 0x0040;
-    } // Bit  8 = C4
-    if id13_field & 0x0080 != 0 {
-        hex_gillham |= 0x4000;
-    } // Bit  7 = A4
-      //if id13_field & 0x0040 != 0 {hex_gillham |= 0x0800;} // Bit  6 = X  or M
-    if id13_field & 0x0020 != 0 {
-        hex_gillham |= 0x0100;
-    } // Bit  5 = B1
-    if id13_field & 0x0010 != 0 {
-        hex_gillham |= 0x0001;
-    } // Bit  4 = D1 or Q
-    if id13_field & 0x0008 != 0 {
-        hex_gillham |= 0x0200;
-    } // Bit  3 = B2
-    if id13_field & 0x0004 != 0 {
-        hex_gillham |= 0x0002;
-    } // Bit  2 = D2
-    if id13_field & 0x0002 != 0 {
-        hex_gillham |= 0x0400;
-    } // Bit  1 = B4
-    if id13_field & 0x0001 != 0 {
-        hex_gillham |= 0x0004;
-    } // Bit  0 = D4
-
-    hex_gillham
-}
-
 /// Even / Odd
 #[derive(Debug, PartialEq, DekuRead, Copy, Clone)]
 #[deku(type = "u8", bits = "1")]
@@ -586,134 +558,10 @@ impl std::fmt::Display for Sign {
         )
     }
 }
-mod mode_ac {
-    // TODO test this module
-    /// convert from mode A hex to 0-4095 index
-    pub fn mode_a_to_index(mode_a: u32) -> u32 {
-        (mode_a & 0x0007)
-            | ((mode_a & 0x0070) >> 1)
-            | ((mode_a & 0x0700) >> 2)
-            | ((mode_a & 0x7000) >> 3)
-    }
-
-    pub fn decode_id13_field(id13_field: u32) -> u32 {
-        let mut hex_gillham: u32 = 0;
-
-        if id13_field & 0x1000 != 0 {
-            hex_gillham |= 0x0010;
-        } // Bit 12 = C1
-        if id13_field & 0x0800 != 0 {
-            hex_gillham |= 0x1000;
-        } // Bit 11 = A1
-        if id13_field & 0x0400 != 0 {
-            hex_gillham |= 0x0020;
-        } // Bit 10 = C2
-        if id13_field & 0x0200 != 0 {
-            hex_gillham |= 0x2000;
-        } // Bit  9 = A2
-        if id13_field & 0x0100 != 0 {
-            hex_gillham |= 0x0040;
-        } // Bit  8 = C4
-        if id13_field & 0x0080 != 0 {
-            hex_gillham |= 0x4000;
-        } // Bit  7 = A4
-          //if id13_field & 0x0040 != 0 {hex_gillham |= 0x0800;} // Bit  6 = X  or M
-        if id13_field & 0x0020 != 0 {
-            hex_gillham |= 0x0100;
-        } // Bit  5 = B1
-        if id13_field & 0x0010 != 0 {
-            hex_gillham |= 0x0001;
-        } // Bit  4 = D1 or Q
-        if id13_field & 0x0008 != 0 {
-            hex_gillham |= 0x0200;
-        } // Bit  3 = B2
-        if id13_field & 0x0004 != 0 {
-            hex_gillham |= 0x0002;
-        } // Bit  2 = D2
-        if id13_field & 0x0002 != 0 {
-            hex_gillham |= 0x0400;
-        } // Bit  1 = B4
-        if id13_field & 0x0001 != 0 {
-            hex_gillham |= 0x0004;
-        } // Bit  0 = D4
-
-        hex_gillham
-    }
-
-    pub fn mode_a_to_mode_c(mode_a: u32) -> Result<u32, &'static str> {
-        let mut five_hundreds: u32 = 0;
-        let mut one_hundreds: u32 = 0;
-
-        // check zero bits are zero, D1 set is illegal; C1,,C4 cannot be Zero
-        if (mode_a & 0xFFFF_8889) != 0 || (mode_a & 0x0000_00F0) == 0 {
-            return Err("Invalid altitude");
-        }
-
-        if mode_a & 0x0010 != 0 {
-            one_hundreds ^= 0x007;
-        } // C1
-        if mode_a & 0x0020 != 0 {
-            one_hundreds ^= 0x003;
-        } // C2
-        if mode_a & 0x0040 != 0 {
-            one_hundreds ^= 0x001;
-        } // C4
-
-        // Remove 7s from OneHundreds (Make 7->5, snd 5->7).
-        if (one_hundreds & 5) == 5 {
-            one_hundreds ^= 2;
-        }
-
-        // Check for invalid codes, only 1 to 5 are valid
-        if one_hundreds > 5 {
-            return Err("Invalid altitude");
-        }
-
-        // if mode_a & 0x0001 {five_hundreds ^= 0x1FF;} // D1 never used for altitude
-        if mode_a & 0x0002 != 0 {
-            five_hundreds ^= 0x0FF;
-        } // D2
-        if mode_a & 0x0004 != 0 {
-            five_hundreds ^= 0x07F;
-        } // D4
-
-        if mode_a & 0x1000 != 0 {
-            five_hundreds ^= 0x03F;
-        } // A1
-        if mode_a & 0x2000 != 0 {
-            five_hundreds ^= 0x01F;
-        } // A2
-        if mode_a & 0x4000 != 0 {
-            five_hundreds ^= 0x00F;
-        } // A4
-
-        if mode_a & 0x0100 != 0 {
-            five_hundreds ^= 0x007;
-        } // B1
-        if mode_a & 0x0200 != 0 {
-            five_hundreds ^= 0x003;
-        } // B2
-        if mode_a & 0x0400 != 0 {
-            five_hundreds ^= 0x001;
-        } // B4
-
-        // Correct order of one_hundreds.
-        if five_hundreds & 1 != 0 && one_hundreds <= 6 {
-            one_hundreds = 6 - one_hundreds;
-        }
-
-        // Check for invalid one_hundres
-        if one_hundreds < 13 {
-            return Err("Invalid one_hundred");
-        }
-
-        Ok((five_hundreds * 5) + one_hundreds - 13)
-    }
-}
 
 /// 13 bit identity code
 #[derive(Debug, PartialEq, DekuRead, Copy, Clone)]
-pub struct IdentityCode(#[deku(reader = "Self::read(deku::rest)")] u16);
+pub struct IdentityCode(#[deku(reader = "Self::read(deku::rest)")] pub u16);
 
 impl IdentityCode {
     fn read(rest: &BitSlice<Msb0, u8>) -> Result<(&BitSlice<Msb0, u8>, u16), DekuError> {
@@ -828,7 +676,7 @@ impl std::fmt::Display for FlightStatus {
 pub struct AC13Field(#[deku(reader = "Self::read(deku::rest)")] pub u32);
 
 impl AC13Field {
-    /// TODO Add unit
+    // TODO Add unit
     fn read(rest: &BitSlice<Msb0, u8>) -> Result<(&BitSlice<Msb0, u8>, u32), DekuError> {
         let (rest, num) = u32::read(rest, (deku::ctx::Endian::Big, deku::ctx::Size::Bits(13)))?;
 
@@ -894,3 +742,26 @@ impl std::fmt::Display for Capability {
         )
     }
 }
+
+const CHAR_LOOKUP: &[u8; 64] = b"#ABCDEFGHIJKLMNOPQRSTUVWXYZ##### ###############0123456789######";
+
+pub(crate) fn aircraft_identification_read(
+    rest: &BitSlice<Msb0, u8>,
+) -> Result<(&BitSlice<Msb0, u8>, String), DekuError> {
+    let mut inside_rest = rest;
+
+    let mut chars = vec![];
+    for _ in 0..=6 {
+        let (for_rest, c) = <u8>::read(inside_rest, deku::ctx::Size::Bits(6))?;
+        if c != 32 {
+            chars.push(c);
+        }
+        inside_rest = for_rest;
+    }
+    let encoded = chars
+        .into_iter()
+        .map(|b| CHAR_LOOKUP[b as usize] as char)
+        .collect::<String>();
+
+    Ok((inside_rest, encoded))
+}
