@@ -1,7 +1,6 @@
 use adsb_deku::adsb::{VerticalRateSource, ME};
 use adsb_deku::deku::prelude::*;
 use adsb_deku::{CPRFormat, Capability, Frame, DF};
-
 use assert_hex::assert_eq_hex;
 use hexlit::hex;
 
@@ -287,6 +286,7 @@ fn testing_df_extendedsquitteraircraftopstatus() {
    Version:            2
    Capability classes: ACAS ARV TS
    Operational modes:  SAF SDA=3
+   NIC-A:              1
    NACp:               10
    GVA:                2
    SIL:                3 (per hour)
@@ -307,6 +307,7 @@ fn testing_df_extendedsquitteraircraftopstatus() {
    Version:            2
    Capability classes: ACAS ARV TS
    Operational modes:  SAF SDA=2
+   NIC-A:              0
    NACp:               9
    GVA:                2
    SIL:                3 (per hour)
@@ -343,7 +344,6 @@ fn testing_airbornepositionbaroaltitude() {
   Altitude:      23650 ft barometric
   CPR type:      Airborne
   CPR odd flag:  even
-  CPR NUCp/NIC:  ?
   CPR latitude:  (87769)
   CPR longitude: (71577)
 "#,
@@ -447,27 +447,10 @@ fn testing_targetstateandstatusinformation() {
     NACp:              10
     NICbaro:           1
     SIL:               3 (per sample)
+    QNH:               1012.8 millibars
 "#,
         resulting_string
     );
-    //        let bytes = hex!("8da2c1bd587ba2adb31799cb802b");
-    //        let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
-    //        let resulting_string = format!("{}", frame);
-    //        assert_eq!(
-    //            r#" Extended Squitter Target state and status (V2) (29/1)
-    //  ICAO Address:  A230D6 (Mode S / ADS-B)
-    //  Air/Ground:    airborne
-    //  Target State and Status:
-    //    Target altitude:   MCP, 33024 ft
-    //    Altimeter setting: 1013.6 millibars
-    //    Active modes:      autopilot VNAV
-    //    ACAS:              operational
-    //    NACp:              10
-    //    NICbaro:           1
-    //    SIL:               3 (per sample)
-    //"#,
-    //            resulting_string
-    //        );
 }
 
 #[test]
@@ -503,6 +486,7 @@ fn testing_issue_01() {
     NACp:              8
     NICbaro:           1
     SIL:               3 (per sample)
+    QNH:               1013.6 millibars
 "#,
         resulting_string
     );
@@ -544,9 +528,9 @@ fn testing_df_21() {
     let resulting_string = format!("{}", frame);
     assert_eq!(
         r#" Comm-B, Identity Reply
-    Comm-B format: unknown format
     ICAO Address:  a95fdc (Mode S / ADS-B)
     Squawk:        6246
+    Comm-B format: unknown format
 "#,
         resulting_string
     );
@@ -578,7 +562,6 @@ fn testing_df_18() {
   Altitude:      700 ft barometric
   CPR type:      Airborne
   CPR odd flag:  odd
-  CPR NUCp/NIC:  ?
   CPR latitude:  (74955)
   CPR longitude: (28998)
 "#,
@@ -642,7 +625,6 @@ fn testing_df_18() {
   Altitude:      2000 ft barometric
   CPR type:      Airborne
   CPR odd flag:  odd
-  CPR NUCp/NIC:  ?
   CPR latitude:  (68677)
   CPR longitude: (31345)
 "#,
@@ -655,7 +637,284 @@ fn testing_df_18() {
     let resulting_string = format!("{}", frame);
     assert_eq!(
         r#" Extended Squitter (Non-Transponder) Reserved for surface system status
-  Address:      adf9ce (ADS-B)
+  Address:       adf9ce (ADS-B)
+  Air/Ground:    airborne?
+"#,
+        resulting_string
+    );
+
+    let bytes = hex!("97CAEEF737FB1341BF58DF19118A");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter (Non-Transponder) Surface position
+  Address:       caeef7 (unknown addressing scheme)
+"#,
+        resulting_string
+    );
+
+    // test github issue #8 (with sample output from dump1090_fa as control)
+    let bytes = hex!("96A4D01FF900210600493075E234");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter (Non-Transponder) Aircraft operational status (surface)
+ Address:       a4d01f (ADS-R)
+ Air/Ground:    airborne?
+ Aircraft Operational Status:
+   Version:            2
+   NIC-A:              0
+   NIC-C:              0
+   NACv:               1
+   Capability classes: L/W=1
+   Operational modes:  SAF SDA=2
+   NACp:               9
+   SIL:                3 (per hour)
+   NICbaro:            0
+   Heading reference:  true north
+"#,
+        resulting_string
+    );
+}
+#[test]
+fn test_emergency() {
+    let bytes = hex!("8dc06800e1108500000000baa81f");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter Emergency/priority status
+  Address:       c06800 (Mode S / ADS-B)
+  Air/Ground:    airborne
+  Squawk:        4016
+  Emergency/priority:    no emergency
+"#,
+        resulting_string
+    );
+}
+
+#[test]
+fn issue_10() {
+    let bytes = hex!("8DA35EBC9B000024B00C0004E897");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter Airspeed and heading, subsonic
+  Address:       a35ebc (Mode S / ADS-B)
+  Air/Ground:    airborne
+  IAS:           292 kt
+  Baro rate:     128 ft/min
+  NACv:          0
+"#,
+        resulting_string
+    );
+}
+
+#[test]
+fn issue_11_12() {
+    let bytes = hex!("8da90a6e000000000000005cab8b");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter No position information
+  Address:       a90a6e (Mode S / ADS-B)
+  Air/Ground:    airborne
+"#,
+        resulting_string
+    );
+
+    let bytes = hex!("92ef92b301154cb9ab09466702c6");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter (Non-Transponder) No position information
+  Address:       ef92b3 (TIS-B)
+  Air/Ground:    airborne?
+"#,
+        resulting_string
+    );
+}
+
+#[test]
+fn fix_issue_unknown() {
+    let bytes = hex!("8d85d792beaf5654b710d87357ee");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter Unknown
+  Address:       85d792 (Mode S / ADS-B)
+  Air/Ground:    airborne
+"#,
+        resulting_string
+    );
+
+    let bytes = hex!("972ae8d6d73e298fcaa6bec4c338");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter (Non-Transponder) Unknown
+  Address:       2ae8d6 (unknown addressing scheme)
+  Air/Ground:    airborne?
+"#,
+        resulting_string
+    );
+}
+
+#[test]
+fn fix_issue_13() {
+    // 1
+    let bytes = hex!("8dab92a2593e0664204c69d8fe84");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter Airborne position (barometric altitude)
+  Address:       ab92a2 (Mode S / ADS-B)
+  Air/Ground:    airborne
+  Altitude:      10600 ft barometric
+  CPR type:      Airborne
+  CPR odd flag:  odd
+  CPR latitude:  (78352)
+  CPR longitude: (19561)
+"#,
+        resulting_string
+    );
+
+    // 2
+    let bytes = hex!("8dab92a299105e93001486608c6d");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter Airborne velocity over ground, subsonic
+  Address:       ab92a2 (Mode S / ADS-B)
+  Air/Ground:    airborne
+  GNSS delta:    -125 ft
+  Heading:       149
+  Speed:         177 kt groundspeed
+  Vertical rate: 256 ft/min barometric
+"#,
+        resulting_string
+    );
+
+    // 3
+    let bytes = hex!("020007a0d08ff4");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Short Air-Air Surveillance
+  ICAO Address:  ab92a2 (Mode S / ADS-B)
+  Air/Ground:    airborne?
+  Altitude:      10600 ft barometric
+"#,
+        resulting_string
+    );
+
+    // 4
+    let bytes = hex!("5dab92a2b04912");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" All Call Reply
+  ICAO Address:  ab92a2 (Mode S / ADS-B)
+  Air/Ground:    airborne
+"#,
+        resulting_string
+    );
+
+    // 4
+    let bytes = hex!("8dab92a2593e0664204c69d8fe84");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter Airborne position (barometric altitude)
+  Address:       ab92a2 (Mode S / ADS-B)
+  Air/Ground:    airborne
+  Altitude:      10600 ft barometric
+  CPR type:      Airborne
+  CPR odd flag:  odd
+  CPR latitude:  (78352)
+  CPR longitude: (19561)
+"#,
+        resulting_string
+    );
+}
+
+#[test]
+fn test_issue_14() {
+    let bytes = hex!("a0001910204d7075d35820c25c0c");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Comm-B, Altitude Reply
+  ICAO Address:  aa6f80 (Mode S / ADS-B)
+  Altitude:      39000 ft
+  Comm-B format: BDS2,0 Aircraft identification
+  Ident:         SWA545
+"#,
+        resulting_string
+    );
+
+    let bytes = hex!("a000171810030a80f6000012bd7b");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Comm-B, Altitude Reply
+  ICAO Address:  aacb19 (Mode S / ADS-B)
+  Altitude:      36000 ft
+  Comm-B format: BDS1,0 Datalink capabilities
+"#,
+        resulting_string
+    );
+}
+
+#[test]
+fn test_issue_09() {
+    let bytes = hex!("a00017b010030a80f60000a0fc1e");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Comm-B, Altitude Reply
+  ICAO Address:  a6c756 (Mode S / ADS-B)
+  Altitude:      37000 ft
+  Comm-B format: BDS1,0 Datalink capabilities
+"#,
+        resulting_string
+    );
+
+    let bytes = hex!("a000179f0000000000000019a524");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Comm-B, Altitude Reply
+  ICAO Address:  a6c756 (Mode S / ADS-B)
+  Altitude:      36975 ft
+  Comm-B format: empty response
+"#,
+        resulting_string
+    );
+}
+
+#[test]
+fn test_issue_16() {
+    let bytes = hex!("a227ed3417826515bebd01707629");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Comm-B, Altitude Reply
+  ICAO Address:  abef98 (Mode S / ADS-B)
+  Altitude:      20300 ft
+  Comm-B format: unknown format
+"#,
+        resulting_string
+    );
+}
+
+#[test]
+fn test_operational_coordination() {
+    let bytes = hex!("9143e8eef79baeeacca522b044bf");
+    let frame = Frame::from_bytes((&bytes, 0)).unwrap().1;
+    let resulting_string = format!("{}", frame);
+    assert_eq!(
+        r#" Extended Squitter (Non-Transponder) Aircraft Operational Coordination
+  Address:       43e8ee (ADS-B)
 "#,
         resulting_string
     );
